__author__ = 'Nate'

# module import
import pystan
import numpy as np
import pandas as pd
import pylab as py
import scipy.stats as st

### DATA PREP ####
feeding_assays = pd.read_csv("/Users/Nate/Documents/FIU/Research/Temp_HerbResponses/Data/Thermal_Curves/Feeding_Assays.csv")
plantTraits = pd.read_csv("/Users/Nate/Documents/FIU/Research/Temp_HerbResponses/Data/Plant_Traits/TempCons_PlantTraits.csv")

# clean traits
plantTraits = plantTraits[['Plant', 'pctC', 'pctN', 'pctP', 'pct.water']]
plantTraits = plantTraits.dropna()
plantTraits.rename(columns = {'Plant' : 'Food_Type'}, inplace = True)

# merge the plant traits with the feeding assays
feeding_assays = pd.merge(feeding_assays, plantTraits, on = 'Food_Type')

# first, assign which to keep
values = ['Ailanthus Webworm', 'Black Swallowtail', 'Canadian melanophia', 'Dogbane Beetle', 'Dogbane Webworm', 'Eastern Tent Caterpillar', 'Elm Argid Sawfly', 'Fall Webworm', 'Locust Sawfly', 'Milkweed Tussock Moth', 'Monarch', 'Spicebush Leafroller', 'Spicebush Swallowtail', 'Tulip Tree Beauty']
feeding2 = feeding_assays[feeding_assays['Herbivore_Common'].isin(values)]

# next, remove the spicebush leafroller + sassafras combination
feeding_Final = feeding2[(feeding2['Herbivore_Common'] != 'Spicebush Leafroller') | (feeding2['Food_Type'] != 'Sassafras albidum')]

# change temperature of 33 to 35
feeding_Final['Temperature'].replace(33, 35, inplace = True)

# remove all dead or incomplete observations
feeding_Final = feeding_Final[feeding_Final['Comments'] != 'DEAD']
feeding_Final = feeding_Final.dropna(subset = ['Herb_RGR'])

# select only observations with positive consumption rate
feeding_Final = feeding_Final[feeding_Final['MassCorr_Consumption_Daily'] > 0]

# standardize temperature
feeding_Final['Temp_Scale'] = (feeding_Final['Temperature'] - feeding_Final['Temperature'].mean()) / feeding_Final['Temperature'].std()

# preliminary plot of the overall data
feeding_Final.boxplot('MassCorr_Consumption_Daily', by = 'Temperature')
feeding_Final.boxplot('MassCorr_Consumption_Daily', by = ['Herbivore_Common', 'Temperature'])

# replace the NaN w/ a species name for the unknown leafroller
feeding_Final['Herbivore_Scientific'].fillna('Unidentified Tortricid', inplace = True)


# create a numeric curveID
feeding_Final['CurveNames'] = feeding_Final['Herbivore_Scientific'] + '-' + feeding_Final['Food_Type']
# sort
feeding_Final.sort('CurveNames', inplace = True)
feeding_Final['CurveID'] = pd.factorize(feeding_Final['CurveNames'])[0] + 1

# remove duplicate values
curve_level = feeding_Final.drop_duplicates(cols = 'CurveID')
curve_level['scaleN'] = (curve_level['pctN'] - curve_level['pctN'].mean()) / curve_level['pctN'].std()
curve_level['scaleP'] = (curve_level['pctP'] - curve_level['pctP'].mean()) / curve_level['pctP'].std()
curve_level['scaleH20'] = (curve_level['pct.water'] - curve_level['pct.water'].mean()) / curve_level['pct.water'].std()


### CONSUMPTION ANALYSIS ####
stanMod = """
data{
    int<lower = 1> N;
    int<lower = 1> J;
    vector[N] Temp;
    vector[N] y;
    int<lower = 1> Curve[N];
    vector[J] pctN;
    vector[J] pctP;
    vector[J] pctH20;
    matrix[3, 3] R;
}
parameters{
    vector[3] beta[J];
    real mu_a;
    real mu_b;
    real mu_c;
    real g_a1;
    real g_a2;
    real g_a3;
    real g_b1;
    real g_b2;
    real g_b3;
    real g_c1;
    real g_c2;
    real g_c3;
    real<lower = 0> sigma[J];
    cov_matrix[3] Tau;
}
transformed parameters{
    vector[N] y_hat;
    vector[N] sd_y;
    vector[3] beta_hat[J];
    for (n in 1:N){
        y_hat[n] <- exp( beta[Curve[n], 1] + beta[Curve[n], 2]*Temp[n] + beta[Curve[n], 3]*pow(Temp[n], 2) );
        sd_y[n] <- sigma[Curve[n]];
    }
    for (j in 1:J){
        beta_hat[j, 1] <- mu_a + g_a1*pctN[j] + g_a2*pctP[j] + g_a3*pctH20[j];
        beta_hat[j, 2] <- mu_b + g_b1*pctN[j] + g_b2*pctP[j] + g_b3*pctH20[j];
        beta_hat[j, 3] <- mu_c + g_c1*pctN[j] + g_c2*pctP[j] + g_c3*pctH20[j];
    }
}
model{
    y ~ normal(y_hat, sd_y);
    for (j in 1:J){
        beta[j] ~ multi_normal_prec(beta_hat[j], Tau);
    }
    // PRIORS
    mu_a ~ normal(0, 1);
    mu_b ~ normal(0, 1);
    mu_c ~ normal(0, 1);
    g_a1 ~ normal(0, 1);
    g_a2 ~ normal(0, 1);
    g_a3 ~ normal(0, 1);
    g_b1 ~ normal(0, 1);
    g_b2 ~ normal(0, 1);
    g_b3 ~ normal(0, 1);
    g_c1 ~ normal(0, 1);
    g_c2 ~ normal(0, 1);
    g_c3 ~ normal(0, 1);
    sigma ~ uniform(0, 100);
    Tau ~ wishart(4, R);
}
"""

# get variables and data
N = len(feeding_Final)
J = feeding_Final['CurveID'].max()

y = feeding_Final['MassCorr_Consumption_Daily']
Curve = feeding_Final['CurveID']


# make data dictionary
dat = {'N' : N,
       'J' : J,
       'y' : y,
       'Temp' : feeding_Final['Temp_Scale'],
       'Curve' : Curve,
       'pctN' : curve_level['scaleN'],
       'pctP' : curve_level['scaleP'],
       'pctH20' : curve_level['scaleH20'],
       'R' : np.identity(3)}

# fit the model!
fit = pystan.stan(model_code=stanMod, data=dat,
                  iter=10000, chains=4, thin = 20)


fit.plot(['mu_a', 'mu_b', 'mu_c'])
fit.plot(['g_a1', 'g_a2', 'g_a3'])
fit.plot(['g_b1', 'g_b2', 'g_b3'])
fit.plot(['g_c1', 'g_c2', 'g_c3'])
py.show()

#print fit




# PLOT GROUP-LEVEL REGRESSION COEFFICIENTS (PLANT TRAITS)
nCoefs = fit.extract(['g_a1', 'g_b1', 'g_c1'])
pCoefs = fit.extract(['g_a2', 'g_b2', 'g_c2'])
wCoefs = fit.extract(['g_a3', 'g_b3', 'g_c3'])

nCoefs = pd.DataFrame({'Intercept' : nCoefs['g_a1'], 'Linear' : nCoefs['g_b1'], 'Quadratic' : nCoefs['g_c1']})
pCoefs = pd.DataFrame({'Intercept' : pCoefs['g_a2'], 'Linear' : pCoefs['g_b2'], 'Quadratic' : pCoefs['g_c2']})
wCoefs = pd.DataFrame({'Intercept' : wCoefs['g_a3'], 'Linear' : wCoefs['g_b3'], 'Quadratic' : wCoefs['g_c3']})

fig = py.figure(figsize = (6.5*3, 5.5))

ax1 = fig.add_subplot(131)
ax1.plot(nCoefs.median(), range(3), 'ko', ms = 10)
ax1.hlines(range(3), nCoefs.quantile(0.025), nCoefs.quantile(0.975), 'k')
ax1.hlines(range(3), nCoefs.quantile(0.1), nCoefs.quantile(0.9), 'k', linewidth = 3)
ax1.axvline(0, linestyle = 'dashed', color  = 'k')
ax1.set_ylim([-0.5, 2.5])
ax1.set_title('% Nitrogen')
ax1.set_yticks(range(3))
ax1.set_yticklabels(['Intercept', 'Exponential', 'Gaussian'])
ax1.set_xticks(np.arange(-0.6, 0.41, 0.2))

ax2 = fig.add_subplot(132, sharey = ax1)
ax2.plot(pCoefs.median(), range(3), 'ko', ms = 10)
ax2.hlines(range(3), pCoefs.quantile(0.025), pCoefs.quantile(0.975), 'k')
ax2.hlines(range(3), pCoefs.quantile(0.1), pCoefs.quantile(0.9), 'k', linewidth = 3)
ax2.axvline(0, linestyle = 'dashed', color  = 'k')
ax2.set_ylim([-0.5, 2.5])
ax2.set_title('% Phosphorus')
ax2.set_xticks(np.arange(-0.8, 0.51, 0.2))
py.setp(ax2.get_yticklabels(), visible = False)

ax3 = fig.add_subplot(133, sharey = ax1)
ax3.plot(nCoefs.median(), range(3), 'ko', ms = 10)
ax3.hlines(range(3), wCoefs.quantile(0.025), wCoefs.quantile(0.975), 'k')
ax3.hlines(range(3), wCoefs.quantile(0.1), wCoefs.quantile(0.9), 'k', linewidth = 3)
ax3.axvline(0, linestyle = 'dashed', color  = 'k')
ax3.set_ylim([-0.5, 2.5])
ax3.set_title('% Water')
py.setp(ax3.get_yticklabels(), visible = False)

fig.subplots_adjust(wspace = 0, hspace = 0)
fig.text(0.5, 0.04, 'Median Coefficient Estimate (80 and 95% CI)', va = 'center', ha = 'center', fontsize = 14)
py.gca().invert_yaxis()
py.savefig('trait_Quants.pdf', bbox_inches = 'tight')
py.show()







## PLOT OVERALL COEFFICIENT QUANTILES
mus = fit.extract(['mu_a', 'mu_b', 'mu_c'])
mus = pd.DataFrame({'Intercept' : mus['mu_a'], 'Linear' : mus['mu_b'], 'Quadratic' : mus['mu_c']})

py.plot(mus.median(), range(3), 'ko', ms = 10)
py.hlines(range(3), mus.quantile(0.025), mus.quantile(0.975), 'k')
py.hlines(range(3), mus.quantile(0.1), mus.quantile(0.9), 'k', linewidth = 3)
py.axvline(0, linestyle = 'dashed', color = 'k')
py.xlabel('Median Coefficient Estimate (80 and 95% CI)')
py.yticks(range(3), ['Intercept', 'Exponential', 'Gaussian'])
py.ylim([-0.5, 2.5])
py.title('Overall Coefficients')
py.gca().invert_yaxis()
py.savefig('mu_Quants_Traits.pdf', bbox_inches = 'tight')
py.show()







## PLOT THE OVERALL DATA WITH REGRESSION LINE

#first, define a prediction function
def predFunc(x, v = 1):
    yhat = np.exp( x[0] + x[1]*xPred + v*x[2]*xPred**2 )
    return pd.Series({'yhat' : yhat})

# next, define a function to return the quantiles at each predicted value
def quantGet(data , q):
    quant = []
    for i in range(len(xPred)):
        val = []
        for j in range(len(data)):
            val.append( data[j][i] )
        quant.append( np.percentile(val, q) )
    return quant

# make a vector of temperatures to predict (and convert to the real temperature scale)
xPred = np.linspace(feeding_Final['Temp_Scale'].min(), feeding_Final['Temp_Scale'].max(), 100)
realTemp = xPred * feeding_Final['Temperature'].std() + feeding_Final['Temperature'].mean()

# make predictions for every chain (in overall effects)
ovPred = mus.apply(predFunc, axis = 1)

# get lower and upper quantiles
ovLower = quantGet(ovPred['yhat'], 2.5)
ovLower80 = quantGet(ovPred['yhat'], 10)
ovUpper80 = quantGet(ovPred['yhat'], 90)
ovUpper = quantGet(ovPred['yhat'], 97.5)

# get median predictions
ovPred = predFunc(mus.median())

# summarize feeding data by curve
curveGroup = feeding_Final.groupby(['CurveNames', 'Temperature'])
curveSumm = curveGroup['MassCorr_Consumption_Daily'].agg({'mean' : np.mean, 'SE' : lambda x: x.std() / np.sqrt(x.count()), 'n' : lambda x: x.count()})

# unstack by temperature
temps = curveSumm.unstack('Temperature')['mean']

# make boxes for boxplot by isolating columns into a list
boxes = [temps.iloc[:,x] for x in range(4)]

# BOXPLOT!
f = py.figure()
ax = f.add_subplot(111)
bp = ax.boxplot(boxes, positions = [20, 25, 30, 35], patch_artist = True, sym = 'o', widths = 1.5)
ax.fill_between(realTemp, ovLower, ovUpper, facecolor = 'lightgreen', edgecolor = 'None', alpha = 0.75, zorder = 2)
ax.fill_between(realTemp, ovLower80, ovUpper80, facecolor = 'lightblue', edgecolor = 'None', alpha = 0.75, zorder = 3)
ax.plot(realTemp, ovLower80, 'blue', lw = 1)
ax.plot(realTemp, ovUpper80, 'blue', lw = 1)
ax.plot(realTemp, ovLower, 'green', lw = 1)
ax.plot(realTemp, ovUpper, 'green', lw = 1)
ax.plot(realTemp, ovPred['yhat'], 'r', lw = 2, zorder = 4)
ax.set_ylabel('Consumption Rate (g per g body mass)')
ax.set_xlabel(u'Temperature (\u00B0C)')
ax.set_xlim([18, 37])
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.spines['left'].set_visible(False)
ax.yaxis.set_ticks_position('none')
ax.xaxis.set_ticks_position('bottom')
ax.yaxis.grid(linestyle = 'solid', color = 'lightgrey', lw = 0.75)
ax.set_axisbelow(True)
py.setp(bp['boxes'], facecolor = 'gold')
py.setp(bp['medians'], color = 'black')
py.setp(bp['whiskers'], linestyle = 'solid', color = 'black')
py.setp(bp['fliers'], mfc = 'gold', mew = 1)
p1 = py.Rectangle((0, 0), 1, 1, fc = 'lightgreen')
p2 = py.Rectangle((0, 0), 1, 1, fc = 'lightblue')
py.legend([p2, p1], ['80% CI', '95% CI'], frameon = False)
py.savefig('mu_Predict.pdf', bbox_inches = 'tight')
py.show()







## CURVE LEVEL PARAMETERS
# get the curve level intercepts, linear, and quadratic coefficients
beta = fit.extract('beta')

aChains = pd.DataFrame(beta['beta'][:,:,0])
bChains = pd.DataFrame(beta['beta'][:,:,1])
cChains = pd.DataFrame(beta['beta'][:,:,2])

# rename the columns
curveNames = curve_level['CurveNames'].unique()
aChains.columns = curveNames
bChains.columns = curveNames
cChains.columns = curveNames

# check autocorrelation of parameter estimates
for i in range(21):
    pd.tools.plotting.autocorrelation_plot(aChains.iloc[:,i])
for i in range(21):
    pd.tools.plotting.autocorrelation_plot(bChains.iloc[:,i])
for i in range(21):
    pd.tools.plotting.autocorrelation_plot(cChains.iloc[:,i])

# check the normal distribution of the parameter estimates
st.probplot(aChains.median(), dist='norm', plot=py)
py.show()
st.probplot(bChains.median(), dist='norm', plot=py)
py.show()
st.probplot(cChains.median(), dist='norm', plot=py)
py.show()
# normality looks great

# next, do residual checks
# get fitted values
fitted = fit.extract('y_hat')
fitted = pd.DataFrame(fitted['y_hat'])

fitMed = fitted.median()

# plot normality of residuals (although this assumption is fairly weak)
st.probplot(fitMed, dist='norm', plot=py)
py.show()
# normality really doesn't look good, but the normality assumption isn't terribly important

# check variance assumptions by extracting the standard deviations, calculating the weighted residuals, and plotting against fitted values
wts = fit.extract('sd_y')
wts = pd.DataFrame(wts['sd_y'])

resids = np.array(y) - np.array(fitMed) / np.array(wts.median() ** 2)

py.plot(fitMed, resids, 'ko')
py.xlabel('Fitted Values')
py.ylabel('Weighted Residuals')
py.axhline(0, linestyle = 'dashed')
py.show()









## CURVE LEVEL COEFFICIENTS PLOT

fig = py.figure(figsize = (6.5*3, 5.5))

ax1 = fig.add_subplot(131)
ax1.plot(aChains.median(), range(21), 'ko', ms = 10)
ax1.hlines(range(21), aChains.quantile(0.025), aChains.quantile(0.975), 'k')
ax1.hlines(range(21), aChains.quantile(0.1), aChains.quantile(0.9), 'k', linewidth = 2.5)
ax1.axvline(0, linestyle = 'dashed', color = 'k')
ax1.set_yticks(range(21))
ax1.set_yticklabels(curveNames)
ax1.set_ylim([-0.5, 20.5])
ax1.set_xticks(np.arange(-1.5, 1.51, 0.5))
ax1.set_title('Intercept (a)')

ax2 = fig.add_subplot(132, sharey = ax1)
ax2.plot(bChains.median(), range(21), 'ko', ms = 10)
ax2.hlines(range(21), bChains.quantile(0.025), bChains.quantile(0.975), 'k')
ax2.hlines(range(21), bChains.quantile(0.1), bChains.quantile(0.9), 'k', linewidth = 2.5)
ax2.axvline(0, linestyle = 'dashed', color = 'k')
ax2.set_ylim([-0.5, 20.5])
ax2.set_title('Exponential (b)')
ax2.set_xticks(np.arange(-1, 1.5, 0.5))
py.setp(ax2.get_yticklabels(), visible = False)

ax3 = fig.add_subplot(133, sharey = ax1)
ax3.plot(cChains.median(), range(21), 'ko', ms = 10)
ax3.hlines(range(21), cChains.quantile(0.025), cChains.quantile(0.975), 'k')
ax3.hlines(range(21), cChains.quantile(0.1), cChains.quantile(0.9), 'k', linewidth = 2.5)
ax3.axvline(0, linestyle = 'dashed', color = 'k')
ax3.set_ylim([-0.5, 20.5])
ax3.set_title('Gaussian (c)')
py.setp(ax3.get_yticklabels(), visible = False)

fig.subplots_adjust(wspace = 0, hspace = 0)
fig.text(0.5, 0.04, 'Median Coefficient Estimate (80 and 95% CI)', ha = 'center', va = 'center', fontsize = 14)
py.gca().invert_yaxis()
py.savefig('curve_Quants_Traits.pdf', bbox_inches = 'tight')
py.show()







## PLOT THE PREDICTIONS AND CI
# group the data and summarize for plotting
group = feeding_Final.groupby(['CurveNames', 'Temperature'])
summary = group['MassCorr_Consumption_Daily'].agg( {'mean' : np.mean, 'SE' : lambda x: x.std() / np.sqrt(x.count()), 'n' : lambda x: x.count()} )
curveplot = [x.replace('-', '\n') for x in curveNames]


f = py.figure(figsize = (20, 20))

for i in range(21):
    ax =f.add_subplot(5, 5, i+1)
    if cChains.quantile(0.975)[i] < 0:
        temp = pd.concat( [aChains.iloc[:,i], bChains.iloc[:,i], cChains.iloc[:,i]], axis = 1 )
        pred = temp.apply(predFunc, axis = 1)
        lower = quantGet(pred['yhat'], 2.5)
        upper = quantGet(pred['yhat'], 97.5)
        ax.fill_between(realTemp, lower, upper, facecolor = 'lightgreen', edgecolor = 'green', alpha = 0.75)
        yhat = predFunc([aChains.iloc[:,i].median(), bChains.iloc[:,i].median(), cChains.iloc[:,i].median()])
        ax.plot(realTemp, yhat['yhat'], 'r', lw = 2)
    elif cChains.quantile(0.9)[i] < 0:
        temp = pd.concat( [aChains.iloc[:,i], bChains.iloc[:,i], cChains.iloc[:,i]], axis = 1 )
        pred = temp.apply(predFunc, axis = 1)
        lower = quantGet(pred['yhat'], 2.5)
        upper = quantGet(pred['yhat'], 97.5)
        ax.fill_between(realTemp, lower, upper, facecolor = 'lightgreen', edgecolor = 'green', alpha = 0.75)
        yhat = predFunc([aChains.iloc[:,i].median(), bChains.iloc[:,i].median(), cChains.iloc[:,i].median()])
        ax.plot(realTemp, yhat['yhat'], 'r', linestyle = 'dotted', lw = 2)
    elif bChains.quantile(0.025)[i] > 0:
        temp = pd.concat( [aChains.iloc[:,i], bChains.iloc[:,i], cChains.iloc[:,i]], axis = 1 )
        pred = temp.apply(predFunc, axis = 1, args = (0,))
        lower = quantGet(pred['yhat'], 2.5)
        upper = quantGet(pred['yhat'], 97.5)
        ax.fill_between(realTemp, lower, upper, facecolor = 'lightgreen', edgecolor = 'green', alpha = 0.75)
        yhat = predFunc([aChains.iloc[:,i].median(), bChains.iloc[:,i].median(), 0])
        ax.plot(realTemp, yhat['yhat'], 'r', lw = 2)
    elif bChains.quantile(0.1)[i] > 0:
        temp = pd.concat( [aChains.iloc[:,i], bChains.iloc[:,i], cChains.iloc[:,i]], axis = 1 )
        pred = temp.apply(predFunc, axis = 1, args = (0,))
        lower = quantGet(pred['yhat'], 2.5)
        upper = quantGet(pred['yhat'], 97.5)
        ax.fill_between(realTemp, lower, upper, facecolor = 'lightgreen', edgecolor = 'green', alpha = 0.75)
        yhat = predFunc([aChains.iloc[:,i].median(), bChains.iloc[:,i].median(), 0])
        ax.plot(realTemp, yhat['yhat'], 'r', linestyle = 'dotted', lw = 2)
    ax.errorbar( summary.ix[curveNames[i]].index,
                 summary.ix[curveNames[i]]['mean'],
                 yerr = summary.ix[curveNames[i]]['SE'],
                 fmt = None, capsize = 0, ecolor = 'black', zorder = 2)
    ax.scatter( summary.ix[curveNames[i]].index,
                summary.ix[curveNames[i]]['mean'],
                s = summary.ix[curveNames[i]]['n']*10,
                c = 'gold', zorder = 3)
    ax.set_title(curveplot[i], fontsize = 10)
    ax.set_xlim([18, 37])
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.spines['left'].set_visible(False)
    ax.yaxis.set_ticks_position('none')
    ax.xaxis.set_ticks_position('bottom')
    ax.yaxis.grid(linestyle = 'solid', color = 'lightgrey', lw = 0.75)
    ax.set_axisbelow(True)

text = f.text(0.1, 0.5, 'Consumption Rate (g per g body mass)', va = 'center', ha = 'center', rotation = 90, fontsize = 14)
f.text(0.5, 0.1, u'Temperature (\u00B0C)', va = 'center', ha = 'center', fontsize= 14)
py.savefig('cons_Panel.pdf', bbox_extra_artists=(text,), bbox_inches = 'tight')
py.show()






#### DAILY TEMPERATURE DATA FOR ANNAPOLIS ###
ann = pd.read_csv('/Users/Nate/Documents/FIU/Research/Temp_HerbResponses/Data/annapolis.csv')
# get year
ann['Year'] = ann['DATE'].apply(lambda x: str(x)[:4])
ann['Month'] = ann['DATE'].apply(lambda x: str(x)[4:6])
ann['Day'] = ann['DATE'].apply(lambda x: str(x)[6:8])
ann['Year'] = ann['Year'].astype('string')
ann['TMIN'] = ann['TMIN'] / 10.
ann['TMAX'] = ann['TMAX'] / 10.
ann['TOBS'] = ann['TOBS'] / 10
ann = ann[ann['TOBS'] > -100]
# date conversion
ann['DateTime'] = pd.to_datetime(ann['DATE'].astype('string'), format = '%Y%m%d')
ann.set_index(ann['DateTime'], inplace = True)

f = py.figure(figsize = (8, 5.5))
ax = f.add_subplot(111)
ax.plot( ann.index.to_pydatetime(), ann['TOBS'], '-', color = 'k', linewidth = 1.5, label = 'Daily Mean')
ax.plot( ann.index.to_pydatetime(), ann['TMIN'], '-', color = 'b', linewidth = 1.5, label = 'Daily Minimum' )
ax.plot( ann.index.to_pydatetime(), ann['TMAX'], '-', color = 'r', linewidth = 1.5, label = 'Daily Maximum' )
ax.set_ylabel(u'Temperature (\u00B0C)')
ax.legend(loc = 'best')
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.spines['left'].set_visible(False)
ax.yaxis.set_ticks_position('none')
ax.xaxis.set_ticks_position('bottom')
ax.xaxis.grid(False)
ax.yaxis.grid(linestyle = 'solid', color = 'lightgrey', lw = 0.75)
ax.set_axisbelow(True)
f.autofmt_xdate()
py.savefig('annapolis_Temps.pdf', bbox_inches = 'tight')
py.show()





### ANNAPOLIS HOURLY TEMPERATURE ###
ann_hourly = pd.read_csv('/Users/Nate/Documents/FIU/Research/Temp_HerbResponses/Data/annapolis_hourly.csv',
                         usecols = ['Date', 'Time', 'DryBulbCelsius'] )

ann_hourly['Time2'] = ann_hourly['Date'].apply(str) + ' ' + ann_hourly['Time'].apply(lambda x: x.astype('string').zfill(4))
ann_hourly['DateTime'] = pd.to_datetime(ann_hourly['Time2'], format = "%Y%m%d %H%M")
ann_hourly.set_index(ann_hourly['DateTime'], inplace = True)



# plot
f = py.figure(figsize = (8,5.5))
ax = f.add_subplot(111)
ann_hourly['DryBulbCelsius'].plot(color = 'k')
ax.set_xlabel('')
ax.set_ylabel(u'Temperature (\u00B0C)')
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.spines['left'].set_visible(False)
ax.yaxis.set_ticks_position('none')
ax.xaxis.set_ticks_position('bottom')
ax.xaxis.grid(False)
ax.yaxis.grid(linestyle = 'solid', color = 'lightgrey', lw = 0.75)
ax.set_axisbelow(True)
py.savefig('annapolis_HourlyTemps.pdf', bbox_inches = 'tight')
py.title('Hourly Temperatures in Annapolis, MD')
py.show()







# PREDICT CONSUMPTION RATES AT HOURLY TEMPERATURES, THEN TAKE THE CUMULATIVE SUM AND CALCULATE % CHANGE FROM BASELINE AT THE END. DO THIS FOR EVERY CHAIN

# for converting the scaled coefficients back to the real values
std = feeding_Final['Temperature'].std()
mu = feeding_Final['Temperature'].mean()


# define a function to calculate the percent change in cumulative consumption for every chain
def pctCalc(x):
    # convert scaled coefficients to the regular range
    a = x[0] - x[1]*mu/std + x[2]*mu**2/std**2
    b = x[1]/std - 2*x[2]*mu/std**2
    c = x[2]/std**2
    # calculate consumption at every time point for baseline
    tPred0 = np.exp(a + b*(ann_hourly['DryBulbCelsius'] + 0) + c*(ann_hourly['DryBulbCelsius'] + 0)**2)
    # calculate consumption at every time point for 3 degree warming
    tPred3 = np.exp(a + b*(ann_hourly['DryBulbCelsius'] + 3) + c*(ann_hourly['DryBulbCelsius'] + 3)**2)
    # calculate consumption at every time point for 5 degree warming
    tPred5 = np.exp(a + b*(ann_hourly['DryBulbCelsius'] + 5) + c*(ann_hourly['DryBulbCelsius'] + 5)**2)
    # calculate cumulative sum for all
    cumsum0 = tPred0.cumsum()
    cumsum3 = tPred3.cumsum()
    cumsum5 = tPred5.cumsum()
    # calculate the pctChange at the final time point
    pct3 = ((cumsum3[-1] - cumsum0[-1])/cumsum0[-1])*100
    pct5 = ((cumsum5[-1] - cumsum0[-1])/cumsum0[-1])*100
    return pd.Series({'pct3' : pct3, 'pct5' : pct5})


# make containers to hold the results for every herbivore-plant pair
pct3mean = []
pct5mean = []
pct3sd = []
pct5sd = []

# for every herbivore-plant pair, calculate the mean % change and SD
for i in range(21):
    tempDF = pd.concat([aChains.iloc[:,i], bChains.iloc[:,i], cChains.iloc[:,i]], axis = 1) # make a dataframe of a, b, and c parameters
    tempDF2 = tempDF.apply(pctCalc, axis = 1)
    pct3mean.append(tempDF2.mean()[0])
    pct5mean.append(tempDF2.mean()[1])
    pct3sd.append(tempDF2.std()[0] / np.sqrt(1000))
    pct5sd.append(tempDF2.std()[1] / np.sqrt(1000))


plot3sd = [([0]*21), pct3sd]
plot5sd = [([0]*21), pct5sd]
# for the one that is negative, switch the positive and negative
plot5sd[0][7] = plot5sd[1][7]
plot5sd[1][7] = 0

# Plot
f = py.figure(figsize = (10, 5.5))
ax = f.add_subplot(111)
ax.bar(np.arange(21), pct3mean, width = 0.35, color = 'y', yerr =plot3sd, ecolor = 'y', label = u'+3\u00B0 C')
ax.bar(np.arange(21) + 0.35, pct5mean, color = 'r', width = 0.35, yerr = plot5sd, ecolor = 'r', label = u'+5\u00B0 C')
ax.set_xlim([-1, 22])
ax.set_xticks(np.arange(21)+0.35)
ax.set_xticklabels( curveNames, rotation = 45, ha = 'right', fontsize = 10 )
ax.set_ylabel('Percent Change in Cumulative Consumption')
ax.axhline(0, color = 'k')
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.spines['left'].set_visible(False)
ax.yaxis.set_ticks_position('none')
ax.xaxis.set_ticks_position('bottom')
ax.xaxis.grid(False)
ax.yaxis.grid(linestyle = 'solid', color = 'lightgrey', lw = 0.75)
ax.set_axisbelow(True)
ax.legend()
py.savefig('cumCons.pdf', bbox_inches = 'tight')
py.show()





